<?php
namespace Factory;

use \Doctrine\DBAL\DriverManager;

class ConnectionFactoryAPI {

    /**
     * Decrypt data through salt.
     *
     * @param string $text Value to be decrypt.
     * @param string $salt Salt used in encryption.
     *
     * @return string
     */
    public static function decrypt($text, $salt)
    {
        $cipher = "AES-256-CBC";
        $decoded = self::decodeEncryptedText($text);
        
        $text = 
        openssl_decrypt(
            $decoded['text'], 
            $cipher, 
            $salt, 
            OPENSSL_NO_PADDING, 
            $decoded['iv']
        );
        $text = trim($text);
        return $text;
    }

    private static function decodeEncryptedText($text){
        $text = base64_decode($text);
        $iv_pos = hexdec($text[0])+1;
        $iv = substr($text, $iv_pos, self::IV_SIZE);
        $text_arr = explode($iv, $text);
        $text_arr[0] = substr($text_arr[0], 1);
        $text = implode('', $text_arr);
        $iv = self::padText($iv, self::IV_SIZE);

        return array('text'=> $text, 'iv'=>$iv);
    }

    const IV_SIZE = 16;
    const TEXT_SIZE = 32;
    

    /**
     * Pad the text
     * 
     * Pad the text based on the normalization of string size to be encrypted
     * 
     * @param $salt
     * @return string
     */
    protected static function padText($text, $size) {
        if (strlen($text) % $size) 
            return str_pad($text, strlen($text) + $size - strlen($text) % $size, "\0");
        return $text;
    }

    protected static function getIv($cipher){
        return openssl_random_pseudo_bytes(openssl_cipher_iv_length($cipher));
    }

    public static function factoryWithEncryptedPassword(array $connectionParams, $salt, $config = null, $eventManager = null, $session= null) {
        if ($connectionParams['isEncrypted']) {
            $connectionParams["password"] = self::decrypt($connectionParams["password"], $salt);
        }
        return DriverManager::getConnection($connectionParams, $config, $eventManager);
    }
}