ZeedhiChart.directive('zhPie', function($timeout, maskEngine, ZhUtil) {
	return {
		restrict: 'A',
		scope: {
			data: '=',
			options: '='
		},
		link: function(scope, ele, attrs) {
			var getWidget = function() {
				return scope.$parent.widget;
			};
			var canvas = ele.find('canvas');
			var text = ele.find('span');
			var data = scope.data;
			var opts = {
				lines: 20,
				angle: 0.5,
				lineWidth: 0.02,
				limitMax: true,
				colorStart: getWidget().lineColor || '#f39c12',
				colorStop: getWidget().lineColor || '#f39c12',
				strokeColor: '#eeeeee',
				highDpiSupport: false,
				pointer: {
					length: 0.70,
					strokeWidth: 0.035,
					color: '#333333'
				}
			};

			ele.click(function() {
				getWidget().setCurrentRow(getWidget().dataSource.data[0]);
			});

			function drawPie() {
				setTextSize();
				setCanvasSize();
				pie.minValue = getWidget().minValue - 1 || -1;
				pie.maxValue = getWidget().maxValue || 100;
				loadPieValueFromDataSource();
				pie.setOptions(opts);
				pie.update(true);
			}

			function setValueText(value) {
				if (getWidget().valueFormat && maskEngine[getWidget().valueFormat + "Format"]) {
					var params = {};
					if (!Util.isUndefined(pie.maxValue)) {
						params.maxValue = pie.maxValue;
					}
					if (!Util.isUndefined(pie.minValue)) {
						params.minValue = pie.minValue + 1;
					}
					if (!Util.isUndefined(getWidget().valuePrecision)) {
						params.precision = getWidget().valuePrecision;
					}
					
					var formatedValue = maskEngine[getWidget().valueFormat + "Format"](value, params);
					getWidget().valueText = formatedValue;
				} else {
					getWidget().valueText = value;
				}
			}

			function setHintText(value) {
				if (getWidget().hintFormat) {
					if (maskEngine[getWidget().hintFormat + "Format"]) {
						// If the given format is a valid maskEngine format uses it to format the values
						var formatedValue = maskEngine[getWidget().hintFormat + "Format"](validateValue(value));
						var formatedMaxValue = maskEngine[getWidget().hintFormat + "Format"](pie.maxValue);
						getWidget().hintText = "(" + formatedValue + "/" + formatedMaxValue + ")";
					} else {
						// @TODO: Evaluate the format function and uses the returned value as hintText
						// var formatedText = ZhUtil.evaluate(getWidget().hintFormat);
						// getWidget().hintText = formatedText;
					}
				} else {
					getWidget().hintText = "(" + value + "/" + pie.maxValue + ")";
				}
			}

			function validateValue(value) {
				var validValue;

				if (value >= pie.maxValue) {
					validValue = pie.maxValue;
				} else if (value <= pie.minValue) {
					validValue = pie.minValue;
				} else {
					validValue = value;
				}

				return validValue;
			}

			function setPieValue(value) {
				var fValue = parseFloat(value);
				var validValue = validateValue(fValue);
				pie.set(validValue);
				setValueText(validValue);
				setHintText(validValue);
			}

			function setTextSize() {
				text.css('font-size', Math.min(ele.innerHeight(), ele.innerWidth()) * 0.15 + 'px');
				text.css('line-height', Math.min(ele.innerHeight(), ele.innerWidth()) * 0.15 + 10 + 'px');
			}

			function setCanvasSize() {
				canvas.attr('width', ele.innerWidth());
				canvas.attr('height', ele.innerHeight() * 0.9);
			}

			var pie = new Donut(canvas[0]);
			pie.minValue = getWidget().minValue - 1 || -1;
			pie.maxValue = getWidget().maxValue || 100;
			pie.animationSpeed = 24;
			$timeout(function() {
				drawPie();
			}, 200);

			if (getWidget().dataSource && getWidget().dataSource.data && getWidget().dataSource.data[0]) {
				pie.value = getWidget().dataSource.data[0][getWidget().valueField];
				setPieValue(pie.value);
			} else {
				pie.value = pie.minValue;
				setPieValue(pie.value);
			}
			getWidget().redraw = function() {
				drawPie();
			};

			$(window).on('resize.' + getWidget().name, function() {
				if (ele.is(":visible")) {
					drawPie();
				} else {
					$(window).off('resize.' + getWidget().name);
				}
			});

			function loadPieValueFromDataSource() {
				var dataSourceRow = getWidget().dataSource.data[0];
				if (dataSourceRow && dataSourceRow[getWidget().valueField]) {
					pie.value = dataSourceRow[getWidget().valueField];
					setPieValue(pie.value);
				}
			}

			scope.$parent.$watch('widget.dataSource.data[0]["' + getWidget().valueField + '"]', function(e) {
				setTextSize();
				setCanvasSize();
				loadPieValueFromDataSource();
			});
		}
	};
});