var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive('zhScroller', ["$timeout", "templateManager", "$document", "ScreenService", "ZhLogger", function($timeout, templateManager, $document, ScreenService, zhLogger) {
	return {
		restrict: 'A',
		link: function($scope, iElement, iAttrs) {
			var expectedHeight = 0;
			var hasAttr = function hasAttr(element, attr) {
				return (typeof element.attr(attr) !== "undefined" && element.attr(attr) !== false) || (typeof element.attr('data-' + attr) !== "undefined" &&
					element.attr('data-' + attr) !== false);
			};

			//Zed - adjust top and bottom dinamically
			var startScroller = function() {
				if (iElement.is(":visible")) {
					var windowHeight = $(window).height();
					var containerTop = iElement.offset().top;
					var containerBottom = 0;
					// if container has margin-top then adjust the top
					var containerMarginTop = parseFloat(iElement.css('margin-top') || window.getComputedStyle(iElement[0], false).marginTop);
					containerTop -= containerMarginTop;

					// defines the maximum height from the container. At least 260px and maximum of half window
					var maxContainerHeight = Math.ceil(windowHeight * 0.5) > 260 ? Math.ceil(windowHeight * 0.5) : 260;

					// verifies if his element has a container fixed on the bottom of page and adjusts the bottom
					var parent = iElement.parent();
					var fixedBottomContainer = parent.children(".container[zh-fixed-bottom]")[0];
					if (fixedBottomContainer) {
						containerBottom = fixedBottomContainer.clientHeight;
					}

					if (iElement.hasClass("has-stripe-footer")) {
						containerBottom += Util.isMobile() ? 50 : 40; // stripe-footer height
					}

					// search for a parent container with position non static or another container[zh-scroller]
					while (parent && parent.css('position') === 'static' &&
						!(parent.hasClass('container') && hasAttr(parent, 'zh-scroller'))) {
						parent = parent.parent();
					}

					// if there is a parent non static then adjusts the top
					if (parent && parent.offset) {
						containerTop = containerTop - parent.offset().top;
					}

					var popup = iElement.closest('.popup-content').length > 0;

					if (!popup) {

						iElement.css('position', 'absolute');
						iElement.css('top', containerTop + 'px');
						iElement.css('bottom', containerBottom + 'px');

					} else {

						iElement.css('top', '0px');
						iElement.css('width', 'auto');
						iElement.css('position', 'relative');

						if (iElement.find('.zh-grid-container').length > 0 ||
							iElement.find('.zh-list-container').length > 0) {
							iElement.css('overflow', 'hidden');
						} else {
							iElement.css('overflow', 'auto');
						}

						var calendar;

						if(Zepto.os.phone) {
							iElement.css('max-height', ( 0.9 * window.innerHeight - 100) + 'px');

						}	else if (iElement.is(".zh-carousel") && iElement.parents(".zh-interval-widget").length) {

							calendar = iElement.find(".zh-calendar-widget");
							iElement.height( calendar.height() + 12);

						} else if (iElement.find('.zh-template-list').length > 0 && iElement.parents(".zh-interval-widget").length) {
							var interval = iElement.parents(".zh-interval-widget");
							calendar = interval.find(".zh-calendar-widget");
							iElement.height( calendar.height() + 12);
						} else if (
							iElement.find('.zh-grid, .zh-new-grid').length > 0 ||
							iElement.find('.zh-template-list').length > 0 ||
							iElement.find('.zh-tab-wrapper').length > 0) {

							//temp-fix: sometimes iElement.offset().top can be negative and it mess things up
							//causing anomalies with the select poupus
							if (containerTop < 0) {
								containerTop = 0;
							}

							if (containerTop === 0) {
								containerTop = getContainerTopForListInsideTabbed(iElement, containerTop);
							}

							// Lists, grids and tree-grids need a fixed height so you can scroll
							iElement.css('height', (maxContainerHeight - containerTop) + 'px');

						} else if(iElement.parents(".zh-carousel").length > 0) {
							iElement.parents(".zh-carousel").css('height', (maxContainerHeight) + 'px');
						} else {
							iElement.css('max-height', (maxContainerHeight - containerTop) + 'px');
						}
					}

					expectedHeight = Zepto.os.phone ? (0.9 * window.innerHeight - 100) : (maxContainerHeight - containerTop);

					// prevent flickering due the reverse order of execution of this function thru the DOM tree
					iElement.animate({
						opacity: 1
					}, 250);

					// prevent bounce when scrolling outside div
					var elementToApplyNonbounce = iAttrs.zhScrollerNonbounceSelector? iElement.find(iAttrs.zhScrollerNonbounceSelector) : iElement;
					if (elementToApplyNonbounce) {
						elementToApplyNonbounce.nonbounce();
					} else {
						zhLogger.warn("nonbouneSelector \"" + iAttrs.nonbounceSelector + "\" was not found!");
					}
					unbindWatch();

					$scope.$broadcast("zh-scrolled-finished", iElement);
				}
			};

			var unbindWatch = $scope.$watch(function() {
				return iElement.is(':visible');
			}, function(value) {
				if (value === true) {
					$timeout(startScroller, 100);
				}
			});

			if(Util.isMobile()) {
				$scope.$watch(function() {
					return ScreenService.isKeyboardOpen() && ScreenService.isPopupOpen();
				}, function(newValue, oldValue) {
					if(ScreenService.isPopupOpen() && ScreenService.getPopupOpen() == $scope.widget) {
						$timeout(function() {
							var popupScroller = iElement.parent("[zh-scroller]");
							if(!popupScroller.length) {
								popupScroller = iElement.parent("[data-zh-scroller]");
							}

							if(popupScroller.length) {
								if(expectedHeight > window.innerHeight - 100) {
									popupScroller.css({
										"height": window.innerHeight - 100
									});
									iElement.css({
										"height": window.innerHeight - 100
									});
								} else {
									popupScroller.css({
										"height": expectedHeight
									});
									iElement.css({
										"height": expectedHeight
									});
								}
								templateManager.updateTemplate();
							}
						}, 400);
					}
				});
			}
		},

		priority: 10
	};

	function getContainerTopForListInsideTabbed(element, containerTop) {
		var tabbedContainer = element.parents(".container.zh-tabbed-container");
		var tabbedHeader = tabbedContainer.find(".zh-header.zh-custom-tabbed-widget");
		if (tabbedHeader.length) {
			containerTop = tabbedHeader.outerHeight();
		}

		return containerTop;
	}
}]);